package com.ibm.wsdl.tck;

import javax.wsdl.*;
import javax.wsdl.extensions.*;
import javax.wsdl.factory.*;
import javax.wsdl.xml.*;
import javax.xml.namespace.*;
import javax.xml.parsers.*;
import junit.framework.*;
import org.w3c.dom.*;

/**
 * This class implements a series of tests used to check
 * javax.wsdl.BindingFault implementations for compliance
 * with the JWSDL specification.
 *
 * @author Matthew J. Duftler (duftler@us.ibm.com)
 */
public class BindingFaultTest extends TestCase
{
  private static final String TEST_CANDIDATE_PROPERTY_NAME =
    "javax.wsdl.factory.WSDLFactoryCandidate";
  private static final String NS_URI_WSDL =
    "http://schemas.xmlsoap.org/wsdl/";

  private Definition definition = null;
  private BindingFault bindingFault = null;

  protected void setUp() throws WSDLException
  {
    // Determine name of JWSDL factory implementation to test.
    String testCandidateName =
      System.getProperty(TEST_CANDIDATE_PROPERTY_NAME);

    if (testCandidateName == null)
    {
      throw new IllegalArgumentException("System property '" +
                                         TEST_CANDIDATE_PROPERTY_NAME +
                                         "' must be specified to run " +
                                         "test suite.");
    }

    WSDLFactory factory = WSDLFactory.newInstance(testCandidateName);

    Assert.assertNotNull("WSDLFactory should not be null.", factory);

    definition = factory.newDefinition();

    Assert.assertNotNull("Definition should not be null.", definition);

    bindingFault = definition.createBindingFault();

    Assert.assertNotNull("BindingFault should not be null.", bindingFault);
  }

  /**
   * Test BindingFault.setDocumentationElement(...) and
   * BindingFault.getDocumentationElement().
   */
  public void testSetGetDocumentationElement()
    throws ParserConfigurationException
  {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

    factory.setNamespaceAware(true);
    factory.setValidating(false);

    DocumentBuilder builder = factory.newDocumentBuilder();
    Document doc = builder.newDocument();
    Element el = doc.createElementNS(NS_URI_WSDL,
                                     "wsdl:documentation");

    bindingFault.setDocumentationElement(el);

    Element el2 = bindingFault.getDocumentationElement();

    Assert.assertTrue("BindingFault.getDocumentationElement() did not " +
                      "return the same Element that was set using " +
                      "BindingFault.setDocumentationElement(...).",
                      el == el2);
  }

  /**
   * Test BindingFault.setName(...) and BindingFault.getName().
   */
  public void testSetGetName()
  {
    String name = "abcdef";

    bindingFault.setName(name);

    String name2 = bindingFault.getName();

    Assert.assertTrue("BindingFault.getName() did not return " +
                      "the same name that was set using " +
                      "BindingFault.setName(...).",
                      name.equals(name2));
  }
}