package com.ibm.wsdl.tck;

import javax.wsdl.*;
import javax.wsdl.extensions.*;
import javax.wsdl.factory.*;
import javax.wsdl.xml.*;
import javax.xml.namespace.*;
import javax.xml.parsers.*;
import junit.framework.*;
import org.w3c.dom.*;

/**
 * This class implements a series of tests used to check
 * javax.wsdl.Output implementations for compliance
 * with the JWSDL specification.
 *
 * @author Matthew J. Duftler (duftler@us.ibm.com)
 */
public class OutputTest extends TestCase
{
  private static final String TEST_CANDIDATE_PROPERTY_NAME =
    "javax.wsdl.factory.WSDLFactoryCandidate";
  private static final String NS_URI_WSDL =
    "http://schemas.xmlsoap.org/wsdl/";

  private Definition definition = null;
  private Output output = null;

  protected void setUp() throws WSDLException
  {
    // Determine name of JWSDL factory implementation to test.
    String testCandidateName =
      System.getProperty(TEST_CANDIDATE_PROPERTY_NAME);

    if (testCandidateName == null)
    {
      throw new IllegalArgumentException("System property '" +
                                         TEST_CANDIDATE_PROPERTY_NAME +
                                         "' must be specified to run " +
                                         "test suite.");
    }

    WSDLFactory factory = WSDLFactory.newInstance(testCandidateName);

    Assert.assertNotNull("WSDLFactory should not be null.", factory);

    definition = factory.newDefinition();

    Assert.assertNotNull("Definition should not be null.", definition);

    output = definition.createOutput();

    Assert.assertNotNull("Output should not be null.", output);
  }

  /**
   * Test Output.setDocumentationElement(...) and
   * Output.getDocumentationElement().
   */
  public void testSetGetDocumentationElement()
    throws ParserConfigurationException
  {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

    factory.setNamespaceAware(true);
    factory.setValidating(false);

    DocumentBuilder builder = factory.newDocumentBuilder();
    Document doc = builder.newDocument();
    Element el = doc.createElementNS(NS_URI_WSDL,
                                     "wsdl:documentation");

    output.setDocumentationElement(el);

    Element el2 = output.getDocumentationElement();

    Assert.assertTrue("Output.getDocumentationElement() did not " +
                      "return the same Element that was set using " +
                      "Output.setDocumentationElement(...).",
                      el == el2);
  }

  /**
   * Test Output.setMessage(...) and Output.getMessage().
   */
  public void testSetGetMessage()
  {
    Message message = definition.createMessage();

    Assert.assertNotNull("Message should not be null.", message);

    output.setMessage(message);

    Message message2 = output.getMessage();

    Assert.assertTrue("Output.getMessage() did not return " +
                      "the same Message that was set using " +
                      "Output.setMessage(...).",
                      message == message2);
  }

  /**
   * Test Output.setName(...) and Output.getName().
   */
  public void testSetGetName()
  {
    String name = "abcdef";

    output.setName(name);

    String name2 = output.getName();

    Assert.assertTrue("Output.getName() did not return " +
                      "the same name that was set using " +
                      "Output.setName(...).",
                      name.equals(name2));
  }
}