/*
 * (c) Copyright IBM Corp 2001, 2005 
 */

package com.ibm.wsdl.tck;

import java.util.*;
import javax.wsdl.*;
import javax.wsdl.extensions.*;
import javax.wsdl.factory.*;
import javax.wsdl.xml.*;
import javax.xml.namespace.*;
import javax.xml.parsers.*;
import junit.framework.*;
import org.w3c.dom.*;

/**
 * This class implements a series of tests used to check
 * javax.wsdl.Part implementations for compliance
 * with the JWSDL specification.
 *
 * @author Matthew J. Duftler (duftler@us.ibm.com)
 */
public class PartTest extends TestCase
{
  private static final String TEST_CANDIDATE_PROPERTY_NAME =
    "javax.wsdl.factory.WSDLFactoryCandidate";
  private static final String NS_URI_WSDL =
    "http://schemas.xmlsoap.org/wsdl/";

  private Definition definition = null;
  private Part part = null;

  protected void setUp() throws WSDLException
  {
    // Determine name of JWSDL factory implementation to test.
    String testCandidateName =
      System.getProperty(TEST_CANDIDATE_PROPERTY_NAME);

    if (testCandidateName == null)
    {
      throw new IllegalArgumentException("System property '" +
                                         TEST_CANDIDATE_PROPERTY_NAME +
                                         "' must be specified to run " +
                                         "test suite.");
    }

    WSDLFactory factory = WSDLFactory.newInstance(testCandidateName);

    Assert.assertNotNull("WSDLFactory should not be null.", factory);

    definition = factory.newDefinition();

    Assert.assertNotNull("Definition should not be null.", definition);

    part = definition.createPart();

    Assert.assertNotNull("Part should not be null.", part);
  }

  /**
   * Test Part.setDocumentationElement(...) and
   * Part.getDocumentationElement().
   */
  public void testSetGetDocumentationElement()
    throws ParserConfigurationException
  {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

    factory.setNamespaceAware(true);
    factory.setValidating(false);

    DocumentBuilder builder = factory.newDocumentBuilder();
    Document doc = builder.newDocument();
    Element el = doc.createElementNS(NS_URI_WSDL,
                                     "wsdl:documentation");

    part.setDocumentationElement(el);

    Element el2 = part.getDocumentationElement();

    Assert.assertTrue("Part.getDocumentationElement() did not " +
                      "return the same Element that was set using " +
                      "Part.setDocumentationElement(...).",
                      el == el2);
  }

  /**
   * Test Part.setElementName(...) and Part.getElementName().
   */
  public void testSetGetElementName()
  {
    QName elementName = new QName("urn:abc", "def");

    part.setElementName(elementName);

    QName elementName2 = part.getElementName();

    Assert.assertTrue("Part.getElementName() did not return " +
                      "the same element name that was set using " +
                      "Part.setElementName(...).",
                      elementName.equals(elementName2));
  }

  /**
   * Test Part.setName(...) and Part.getName().
   */
  public void testSetGetName()
  {
    String name = "abcdef";

    part.setName(name);

    String name2 = part.getName();

    Assert.assertTrue("Part.getName() did not return " +
                      "the same name that was set using " +
                      "Part.setName(...).",
                      name.equals(name2));
  }

  /**
   * Test Part.setTypeName(...) and Part.getTypeName().
   */
  public void testSetGetTypeName()
  {
    QName typeName = new QName("urn:abc", "def");

    part.setTypeName(typeName);

    QName typeName2 = part.getTypeName();

    Assert.assertTrue("Part.getTypeName() did not return " +
                      "the same type name that was set using " +
                      "Part.setTypeName(...).",
                      typeName.equals(typeName2));
  }
}