/*
 * (c) Copyright IBM Corp 2001, 2005 
 */

package com.ibm.wsdl.tck;

import java.util.*;
import javax.wsdl.*;
import javax.wsdl.extensions.*;
import javax.wsdl.factory.*;
import javax.wsdl.xml.*;
import javax.xml.namespace.*;
import javax.xml.parsers.*;
import junit.framework.*;
import org.w3c.dom.*;

/**
 * This class implements a series of tests used to check
 * javax.wsdl.Port implementations for compliance
 * with the JWSDL specification.
 *
 * @author Matthew J. Duftler (duftler@us.ibm.com)
 */
public class PortTest extends TestCase
{
  private static final String TEST_CANDIDATE_PROPERTY_NAME =
    "javax.wsdl.factory.WSDLFactoryCandidate";
  private static final String NS_URI_WSDL =
    "http://schemas.xmlsoap.org/wsdl/";

  private Definition definition = null;
  private Port port = null;

  protected void setUp() throws WSDLException
  {
    // Determine name of JWSDL factory implementation to test.
    String testCandidateName =
      System.getProperty(TEST_CANDIDATE_PROPERTY_NAME);

    if (testCandidateName == null)
    {
      throw new IllegalArgumentException("System property '" +
                                         TEST_CANDIDATE_PROPERTY_NAME +
                                         "' must be specified to run " +
                                         "test suite.");
    }

    WSDLFactory factory = WSDLFactory.newInstance(testCandidateName);

    Assert.assertNotNull("WSDLFactory should not be null.", factory);

    definition = factory.newDefinition();

    Assert.assertNotNull("Definition should not be null.", definition);

    port = definition.createPort();

    Assert.assertNotNull("Port should not be null.", port);
  }

  /**
   * Test Port.setBinding(...) and Port.getBinding().
   */
  public void testSetGetBinding()
  {
    Binding binding = definition.createBinding();

    Assert.assertNotNull("Binding should not be null.", binding);

    port.setBinding(binding);

    Binding binding2 = port.getBinding();

    Assert.assertTrue("Port.getBinding() did not return " +
                      "the same Binding that was set using " +
                      "Port.setBinding(...).",
                      binding == binding2);
  }

  /**
   * Test Port.setDocumentationElement(...) and
   * Port.getDocumentationElement().
   */
  public void testSetGetDocumentationElement()
    throws ParserConfigurationException
  {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

    factory.setNamespaceAware(true);
    factory.setValidating(false);

    DocumentBuilder builder = factory.newDocumentBuilder();
    Document doc = builder.newDocument();
    Element el = doc.createElementNS(NS_URI_WSDL,
                                     "wsdl:documentation");

    port.setDocumentationElement(el);

    Element el2 = port.getDocumentationElement();

    Assert.assertTrue("Port.getDocumentationElement() did not " +
                      "return the same Element that was set using " +
                      "Port.setDocumentationElement(...).",
                      el == el2);
  }

  /**
   * Test Port.setName(...) and Port.getName().
   */
  public void testSetGetName()
  {
    String name = "abcdef";

    port.setName(name);

    String name2 = port.getName();

    Assert.assertTrue("Port.getName() did not return " +
                      "the same name that was set using " +
                      "Port.setName(...).",
                      name.equals(name2));
  }
}